/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.common.binaryfile;

import static org.mockito.Mockito.*;

import java.io.InputStream;
import java.util.concurrent.CompletableFuture;
import java.util.function.BiFunction;
import java.util.function.Function;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.common.binaryfile.BinaryFileTestFactory.TestRequestType;
import de.ozgcloud.common.binaryfile.BinaryFileTestFactory.TestResponseType;
import io.grpc.stub.CallStreamObserver;
import io.grpc.stub.StreamObserver;

class StreamExclusiveFileSenderTest {

	@Mock
	private BiFunction<byte[], Integer, TestRequestType> chunkBuilder;
	@Mock
	private InputStream inputStream;
	@Mock
	private Function<StreamObserver<TestResponseType>, CallStreamObserver<TestRequestType>> reqObserverBuilder;
	@Spy
	@InjectMocks
	private StreamExclusiveFileSender<TestRequestType, TestResponseType> fileSender;

	@Nested
	class TestSend {

		private final CompletableFuture<TestResponseType> resultFuture = CompletableFuture.completedFuture(new TestResponseType());
		@Captor
		private ArgumentCaptor<Runnable> onReadyHandlerCaptor;
		@Mock
		private BinaryFileUploadStreamObserver<TestRequestType, TestResponseType> responseObserver;

		@BeforeEach
		void init() {
			doReturn(resultFuture).when(fileSender).getResultFuture();
		}

		@SuppressWarnings("rawtypes")
		@Test
		void shouldCreateResponseObserver() {
			try (MockedStatic<BinaryFileUploadStreamObserver> mocked = mockStatic(BinaryFileUploadStreamObserver.class)) {
				doNothing().when(fileSender).sendNext();

				fileSender.send();

				mocked.verify(() -> BinaryFileUploadStreamObserver.create(same(resultFuture), onReadyHandlerCaptor.capture()));
				verifyCallsSendNext(onReadyHandlerCaptor.getValue());
			}
		}

		@SuppressWarnings("rawtypes")
		@Test
		void shouldBuildRequestObserver() {
			try (MockedStatic<BinaryFileUploadStreamObserver> mocked = mockStatic(BinaryFileUploadStreamObserver.class)) {
				mocked.when(() -> BinaryFileUploadStreamObserver.create(any(), any())).thenReturn(responseObserver);

				fileSender.send();

				verify(reqObserverBuilder).apply(responseObserver);
			}
		}

		private void verifyCallsSendNext(Runnable runnable) {
			runnable.run();
			verify(fileSender).sendNext();
		}
	}

	@Nested
	class TestCommunicateEndOfTransfer {

		@Mock
		private CallStreamObserver<TestRequestType> requestObserver;

		@Test
		void shouldCallOnCompleted() {
			ReflectionTestUtils.setField(fileSender, "requestObserver", requestObserver);

			fileSender.communicateEndOfTransfer();

			verify(requestObserver).onCompleted();
		}
	}
}
